from datetime import datetime
from typing import Any

from pydantic import BaseModel, Field, computed_field

from maplestory.apis.guild import get_basic_info_by_id, get_guild_id
from maplestory.models.guild import GuildBasic
from maplestory.models.guild.basic import GuildSkill
from maplestory.utils.kst import yesterday


class Guild(BaseModel):
    name: str = Field(frozen=True)
    world: str = Field(frozen=True)

    def model_post_init(self, __context: Any) -> None:
        basic_info = get_basic_info(self.name, self.world)
        # basic 속성의 모든 필드를 반복하며 상위 객체에 설정합니다.
        for field_name in basic_info.model_fields_set:
            value = getattr(basic_info, field_name)
            setattr(self, field_name, value)

        # Value error, "Guild" object has no field "date" [type=value_error, input_value={'name': '리더', 'world': '스카니아'}, input_type=dict]
        # 이런 에러가 발생함.
        # 모든 등록하려는 attributes를 미리 name, world 밑에 등록해놔야 하는 건데,
        # 그러면 그냥 property 추가하는 것과 다를 바 없어짐.

    # @computed_field
    # def id(self) -> str:
    #     return get_guild_id(self.name, self.world)

    # def oguild_id(self) -> str:
    #     return self.id

    # @computed_field(repr=False)
    # def basic(self) -> GuildBasic:
    #     return get_basic_info(self.name, self.world)

    # @property
    # def level(self) -> int:
    #     return self.basic.level

    # @property
    # def point(self) -> int:
    #     return self.basic.point

    # @property
    # def fame(self) -> int:
    #     return self.basic.fame

    # @property
    # def member_count(self) -> int:
    #     return self.basic.member_count

    # @property
    # def members(self) -> list[str]:
    #     # TODO: Add character model
    #     return self.basic.members

    # @property
    # def master_name(self) -> str:
    #     return self.basic.master_name

    # @property
    # def master(self) -> str:
    #     # TODO: Add character model
    #     return self.basic.master_name

    # @property
    # def master_character(self) -> str:
    #     # TODO: Add character model
    #     return self.basic.master_character

    # @property
    # def skills(self) -> list[GuildSkill]:
    #     return self.basic.skills

    # @property
    # def noblesse_skills(self) -> list[GuildSkill]:
    #     return self.basic.noblesse_skills

    # @property
    # def mark(self) -> str:
    #     return self.basic.mark

    # @property
    # def custom_mark(self) -> str:
    #     return self.basic.custom_mark


def get_basic_info(
    guild_name: str,
    world_name: str,
    date: datetime = yesterday(),
) -> GuildBasic:
    """
    길드 기본 정보를 조회합니다.
    Fetches the basic information of the guild.

    Args:
        guild_name : 길드 명. The name of the guild.
        world_name : 월드 명. The name of the world.
        date : 조회 기준일(KST). Reference date for the query (KST).

    Returns:
        GuildBasic: 길드의 기본 정보. The basic information of the guild.
    """

    guild_id = get_guild_id(guild_name, world_name)
    return get_basic_info_by_id(guild_id, date)


def get_basic_info_in_details(
    guild_name: str,
    world_name: str,
    date: datetime = yesterday(),
) -> GuildBasic:
    """
    길드 기본 정보를 조회합니다.
    Fetches the basic information of the guild.

    Args:
        guild_name : 길드명. The name of the guild.
        world_name : 월드명. The name of the world.
        date : 조회 기준일(KST). Reference date for the query (KST).

    Returns:
        GuildBasic: 길드의 기본 정보. The basic information of the guild.
    """

    guild_basic: GuildBasic = get_basic_info(guild_name, world_name, date)

    from maplestory.services.character import get_basic_character_info

    # Add master character info
    guild_basic.master = get_basic_character_info(guild_basic.master_name, date)

    # TODO: Add members character info

    return guild_basic



"""캐릭터 정보 조회 API를 제공하는 모듈입니다.

Note:
    - 2023년 12월 21일부터 데이터를 조회할 수 있습니다.
    - 캐릭터 정보 조회 API는 일자별 데이터로 매일 오전 1시부터 전일 데이터 조회가 가능합니다.
      (예를 들어, 12월 22일 데이터를 조회하면 22일 00시부터 23일의 00시 사이의 데이터가 조회됩니다.)
    - 게임 콘텐츠 변경으로 ocid가 변경될 수 있습니다. ocid 기반 서비스 갱신 시 유의해 주시길 바랍니다.
"""

from datetime import datetime

import maplestory.utils.date as dates
import maplestory.utils.kst as kst
from maplestory.models.character import (
    Ability,
    AndroidEquipment,
    BeautyEquipment,
    CashitemEquipment,
    CharacterBasic,
    CharacterDojang,
    CharacterEquipment,
    CharacterLinkSkill,
    CharacterPet,
    CharacterSkill,
    CharacterStat,
    HexaMatrix,
    HexaMatrixStat,
    HyperStat,
    Popularity,
    Propensity,
    SetEffect,
    SymbolEquipment,
    VMatrix,
)
from maplestory.models.character.character import CharacterId
from maplestory.utils.network import fetch


def get_character_ocid(
    character_name: str,
) -> str:
    """캐릭터 식별자(ocid)를 조회합니다.

    Args:
        character_name : 캐릭터 이름.

    Returns:
        str: 캐릭터 식별자(ocid)
    """

    path = "/maplestory/v1/id"
    query = {
        "character_name": character_name,
    }
    response = fetch(path, query)

    return CharacterId.model_validate(response).ocid


def get_basic_character_info_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterBasic:
    """캐릭터의 기본 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterBasic: 캐릭터의 기본 정보.
                        The basic information of the character.
    """

    path = "/maplestory/v1/character/basic"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterBasic.model_validate(response)


def get_popularity_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> Popularity:
    """
    캐릭터의 인기도 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterPopularity: 캐릭터의 인기도 정보.
    """

    path = "/maplestory/v1/character/popularity"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return Popularity.model_validate(response)


def get_character_stat_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterStat:
    """
    캐릭터의 종합능력치 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterStat: 캐릭터의 종합능력치 정보.
    """

    path = "/maplestory/v1/character/stat"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterStat.model_validate(response)


def get_character_hyper_stat_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> HyperStat:
    """
    캐릭터의 하이퍼스탯 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterHyperStat: 캐릭터의 하이퍼스탯 정보.
    """

    path = "/maplestory/v1/character/hyper-stat"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return HyperStat.model_validate(response)


def get_character_propensity_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> Propensity:
    """캐릭터의 성향 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterPropensity: 캐릭터의 성향 정보.
    """

    path = "/maplestory/v1/character/propensity"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return Propensity.model_validate(response)


def get_character_ability_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> Ability:
    """캐릭터의 어빌리티 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterAbility: 캐릭터의 어빌리티 정보.
    """

    path = "/maplestory/v1/character/ability"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return Ability.model_validate(response)


def get_character_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterEquipment:
    """장착한 장비 중 캐시 장비를 제외한 나머지 장비 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterItemEquipment: 캐릭터의 장비 정보.
    """

    path = "/maplestory/v1/character/item-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterEquipment.model_validate(response)


def get_character_cashitem_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CashitemEquipment:
    """
    장착한 캐시 장비 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterCashitemEquipment: 캐릭터의 캐시 장비 정보.
    """

    path = "/maplestory/v1/character/cashitem-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CashitemEquipment.model_validate(response)


def get_character_symbol_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> SymbolEquipment:
    """장착한 심볼 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterSymbolEquipment: 캐릭터의 심볼 장비 정보.
    """

    path = "/maplestory/v1/character/symbol-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return SymbolEquipment.model_validate(response)


def get_character_set_effect_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> SetEffect:
    """적용받고 있는 세트 효과 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterSetEffect: 캐릭터의 세트 효과 정보.
    """

    path = "/maplestory/v1/character/set-effect"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return SetEffect.model_validate(response)


def get_character_beauty_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> BeautyEquipment:
    """캐릭터 헤어, 성형, 피부 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterBeautyEquipment: 캐릭터의 미용 장비 정보.
    """

    path = "/maplestory/v1/character/beauty-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return BeautyEquipment.model_validate(response)


def get_character_android_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> AndroidEquipment:
    """장착한 안드로이드 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterAndroidEquipment: 캐릭터의 안드로이드 장비 정보.
    """

    path = "/maplestory/v1/character/android-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return AndroidEquipment.model_validate(response)


def get_character_pet_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterPet:
    """장착한 펫 및 펫 스킬, 장비 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterPetEquipment: 캐릭터의 펫 장비 정보.
    """

    path = "/maplestory/v1/character/pet-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterPet.model_validate(response)


def get_character_skill_by_ocid(
    character_ocid: str,
    skill_grade: int,
    date: datetime = kst.yesterday(),
) -> CharacterSkill:
    """
    캐릭터 스킬과 하이퍼 스킬 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        skill_grade : 조회하고자 하는 전직 차수.
        date : 조회 기준일 (KST).

    Returns:
        CharacterSkill: 캐릭터의 스킬 정보.
    """

    path = "/maplestory/v1/character/skill"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
        "character_skill_grade": skill_grade,
    }
    response = fetch(path, query)

    # NOTE: 6차 전직을 하지 않았을 경우, 아래와 같이 character_skill_grade가 None으로 날아온다.
    # response = {
    #     'date': '2024-01-19T00:00+09:00',
    #     'character_class': '아델',
    #     'character_skill_grade': None,
    #     'character_skill': []
    # }
    if response.get("character_skill_grade") is None:
        response["character_skill_grade"] = skill_grade

    return CharacterSkill.model_validate(response)


def get_character_link_skill_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterLinkSkill:
    """장착 링크 스킬 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterLinkSkill: 캐릭터의 링크 스킬 정보.
    """

    path = "/maplestory/v1/character/link-skill"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterLinkSkill.model_validate(response)


def get_character_vmatrix_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> VMatrix:
    """V매트릭스 슬롯 정보와 장착한 V코어 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterVMatrix: 캐릭터의 V매트릭스 정보.
    """

    path = "/maplestory/v1/character/vmatrix"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return VMatrix.model_validate(response)


def get_character_hexamatrix_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> HexaMatrix:
    """HEXA 매트릭스에 장착한 HEXA 코어 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterHexaMatrix: 캐릭터의 HEXA 매트릭스 정보.
    """

    path = "/maplestory/v1/character/hexamatrix"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return HexaMatrix.model_validate(response)


def get_character_hexamatrix_stat_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> HexaMatrixStat:
    """HEXA 매트릭스에 설정한 HEXA 스탯 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterHexaMatrixStat: 캐릭터의 HEXA 매트릭스 스탯 정보.
    """

    path = "/maplestory/v1/character/hexamatrix-stat"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return HexaMatrixStat.model_validate(response)


def get_character_dojang_record_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterDojang:
    """캐릭터 무릉도장 최고 기록 정보를 조회합니다.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
        date : 조회 기준일 (KST).

    Returns:
        CharacterDojang: 캐릭터의 무릉도장 최고 기록 정보.
    """

    path = "/maplestory/v1/character/dojang"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterDojang.model_validate(response)



"""

Note:
    - 2023년 12월 21일부터 데이터를 조회할 수 있습니다.
        Data can be queried from December 21, 2023.
    - 캐릭터 정보 조회 API는 일자별 데이터로 매일 오전 1시부터 전일 데이터 조회가 가능합니다. (예를 들어, 12월 22일 데이터를 조회하면 22일 00시부터 23일의 00시 사이의 데이터가 조회됩니다.)
        The character information query API provides daily data, and data for the previous day can be queried from 1 AM every day. (For example, if you query data for December 22, you will get data from 00:00 on the 22nd to 00:00 on the 23rd.)
    - 게임 콘텐츠 변경으로 ocid가 변경될 수 있습니다. ocid 기반 서비스 갱신 시 유의해 주시길 바랍니다.
        The ocid may change due to game content changes. Please be careful when updating services based on ocid.
"""
from datetime import datetime

from rich import print

import maplestory.utils.date as dates
import maplestory.utils.kst as kst
from maplestory.models.character import (
    Ability,
    AndroidEquipment,
    BeautyEquipment,
    CashitemEquipment,
    CharacterBasic,
    CharacterDojang,
    CharacterEquipment,
    CharacterId,
    CharacterLinkSkill,
    CharacterPet,
    CharacterSkill,
    CharacterStat,
    HexaMatrix,
    HexaMatrixStat,
    HyperStat,
    Popularity,
    Propensity,
    SetEffect,
    SymbolEquipment,
    VMatrix,
)
from maplestory.utils.network import fetch


def get_basic_character_info_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterBasic:
    """
    캐릭터의 기본 정보를 조회합니다.
    Fetches the basic information of a character.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterBasic: 캐릭터의 기본 정보.
                        The basic information of the character.

    Note:
        - 2023년 12월 21일부터 데이터를 조회할 수 있습니다.
          Data can be queried from December 21, 2023.
        - 캐릭터 정보 조회 API는 일자별 데이터로 매일 오전 1시부터 전일 데이터 조회가 가능합니다. (예를 들어, 12월 22일 데이터를 조회하면 22일 00시부터 23일의 00시 사이의 데이터가 조회됩니다.)
          The character information query API provides daily data, and data for the previous day can be queried from 1 AM every day. (For example, if you query data for December 22, you will get data from 00:00 on the 22nd to 00:00 on the 23rd.)
        - 게임 콘텐츠 변경으로 ocid가 변경될 수 있습니다. ocid 기반 서비스 갱신 시 유의해 주시길 바랍니다.
          The ocid may change due to game content changes. Please be careful when updating services based on ocid.
    """
    path = "/maplestory/v1/character/basic"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterBasic.model_validate(response)


def get_popularity_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> Popularity:
    """
    캐릭터의 인기도 정보를 조회합니다.
    Fetches the popularity information of a character.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterPopularity: 캐릭터의 인기도 정보.
                             The popularity information of the character.
    """
    path = "/maplestory/v1/character/popularity"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return Popularity(
        date=api_response.get("date"),
        popularity=api_response.get("popularity"),
    )


def get_character_stat_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterStat:
    """
    캐릭터의 종합능력치 정보를 조회합니다.
    Fetches the comprehensive ability information of a character.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterStat: 캐릭터의 종합능력치 정보.
                       The comprehensive ability information of the character.
    """
    path = "/maplestory/v1/character/stat"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterStat.model_validate(response)


def get_character_hyper_stat_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> HyperStat:
    """
    캐릭터의 하이퍼스탯 정보를 조회합니다.
    Fetches the hyper stat information of a character.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterHyperStat: 캐릭터의 하이퍼스탯 정보.
                            The hyper stat information of the character.
    """
    path = "/maplestory/v1/character/hyper-stat"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return HyperStat.model_validate(response)


def get_character_propensity_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> Propensity:
    """
    캐릭터의 성향 정보를 조회합니다.
    Fetches the propensity information of a character.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterPropensity: 캐릭터의 성향 정보.
                             The propensity information of the character.
    """
    path = "/maplestory/v1/character/propensity"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return Propensity.model_validate(response)


def get_character_ability_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> Ability:
    """
    캐릭터의 어빌리티 정보를 조회합니다.
    Fetches the ability information of a character.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterAbility: 캐릭터의 어빌리티 정보.
                          The ability information of the character.
    """
    path = "/maplestory/v1/character/ability"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return Ability.model_validate(response)


def get_character_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterEquipment:
    """
    장착한 장비 중 캐시 장비를 제외한 나머지 장비 정보를 조회합니다.
    Fetches the information of equipped items excluding cash items.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterItemEquipment: 캐릭터의 장비 정보.
                                The equipment information of the character.
    """
    path = "/maplestory/v1/character/item-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterEquipment.model_validate(response)


def get_character_cashitem_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CashitemEquipment:
    """
    장착한 캐시 장비 정보를 조회합니다.
    Fetches the information of equipped cash items.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterCashitemEquipment: 캐릭터의 캐시 장비 정보.
                                    The cash item equipment information of the character.
    """
    path = "/maplestory/v1/character/cashitem-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    try:
        _ = CashitemEquipment.model_validate(response)
    except Exception as e:
        print(e)
        print(f"{api_response = }")

    return CashitemEquipment.model_validate(response)


def get_character_symbol_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> SymbolEquipment:
    """
    장착한 심볼 정보를 조회합니다.
    Fetches the equipped symbol information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterSymbolEquipment: 캐릭터의 심볼 장비 정보.
                                  The symbol equipment information of the character.
    """
    path = "/maplestory/v1/character/symbol-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return SymbolEquipment.model_validate(response)


def get_character_set_effect_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> SetEffect:
    """
    적용받고 있는 세트 효과 정보를 조회합니다.
    Fetches the applied set effect information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterSetEffect: 캐릭터의 세트 효과 정보.
                            The set effect information of the character.
    """
    path = "/maplestory/v1/character/set-effect"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return SetEffect.model_validate(response)


def get_character_beauty_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> BeautyEquipment:
    """
    캐릭터 헤어, 성형, 피부 정보를 조회합니다.
    Fetches the character's hair, plastic surgery, and skin information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterBeautyEquipment: 캐릭터의 미용 장비 정보.
                                  The beauty equipment information of the character.
    """
    path = "/maplestory/v1/character/beauty-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return BeautyEquipment.model_validate(response)


def get_character_android_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> AndroidEquipment:
    """
    장착한 안드로이드 정보를 조회합니다.
    Fetches the equipped android information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterAndroidEquipment: 캐릭터의 안드로이드 장비 정보.
                                   The android equipment information of the character.
    """
    path = "/maplestory/v1/character/android-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return AndroidEquipment.model_validate(response)


def get_character_pet_equipment_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterPet:
    """
    장착한 펫 및 펫 스킬, 장비 정보를 조회합니다.
    Fetches the equipped pet and pet skill, equipment information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterPetEquipment: 캐릭터의 펫 장비 정보.
                               The pet equipment information of the character.
    """
    path = "/maplestory/v1/character/pet-equipment"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterPet.model_validate(response)


def get_character_skill_by_ocid(
    character_ocid: str,
    skill_grade: int,
    date: datetime = kst.yesterday(),
) -> CharacterSkill:
    """
    캐릭터 스킬과 하이퍼 스킬 정보를 조회합니다.
    Fetches the character's skill and hyper skill information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        skill_grade : 조회하고자 하는 전직 차수.
                      The job advancement grade to query.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterSkill: 캐릭터의 스킬 정보.
                        The skill information of the character.
    """
    path = "/maplestory/v1/character/skill"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
        "character_skill_grade": skill_grade,
    }
    response = fetch(path, query)

    # print(f"api_response: {api_response}")
    # FIXME: 6차 전직을 하지 않았을 경우, 아래와 같이 결과가 온다.
    # {'date': '2024-01-19T00:00+09:00', 'character_class': '아델',
    #  'character_skill_grade': None, 'character_skill': []}
    if api_response.get("character_skill_grade") is None:
        api_response["character_skill_grade"] = skill_grade

    return CharacterSkill.model_validate(response)


def get_character_link_skill_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterLinkSkill:
    """
    장착 링크 스킬 정보를 조회합니다.
    Fetches the equipped link skill information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterLinkSkill: 캐릭터의 링크 스킬 정보.
                            The link skill information of the character.
    """
    path = "/maplestory/v1/character/link-skill"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterLinkSkill.model_validate(response)


def get_character_vmatrix_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> VMatrix:
    """
    V매트릭스 슬롯 정보와 장착한 V코어 정보를 조회합니다.
    Fetches the VMatrix slot information and the equipped VCore information.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterVMatrix: 캐릭터의 V매트릭스 정보.
                          The VMatrix information of the character.
    """
    path = "/maplestory/v1/character/vmatrix"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return VMatrix.model_validate(response)


def get_character_hexamatrix_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> HexaMatrix:
    """
    HEXA 매트릭스에 장착한 HEXA 코어 정보를 조회합니다.
    Fetches the HEXA core information equipped in the HEXA matrix.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterHexaMatrix: 캐릭터의 HEXA 매트릭스 정보.
                             The HEXA matrix information of the character.
    """
    path = "/maplestory/v1/character/hexamatrix"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return HexaMatrix.model_validate(response)


def get_character_hexamatrix_stat_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> HexaMatrixStat:
    """
    HEXA 매트릭스에 설정한 HEXA 스탯 정보를 조회합니다.
    Fetches the HEXA stat information set in the HEXA matrix.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterHexaMatrixStat: 캐릭터의 HEXA 매트릭스 스탯 정보.
                                    The HEXA matrix stat information of the character.
    """
    path = "/maplestory/v1/character/hexamatrix-stat"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return HexaMatrixStat.model_validate(response)


def get_character_dojang_record_by_ocid(
    character_ocid: str,
    date: datetime = kst.yesterday(),
) -> CharacterDojang:
    """
    캐릭터 무릉도장 최고 기록 정보를 조회합니다.
    Fetches the highest record information of the character's Mu Lung Dojang.

    Args:
        character_ocid : 캐릭터의 식별자(ocid).
                         The identifier (ocid) of the character.
        date : 조회 기준일 (KST).
                         Reference date for the query (KST).

    Returns:
        CharacterDojang: 캐릭터의 무릉도장 최고 기록 정보.
                         The highest record information of the character's Mu Lung Dojang.
    """
    path = "/maplestory/v1/character/dojang"
    query = {
        "ocid": character_ocid,
        "date": dates.to_string(date),
    }
    response = fetch(path, query)

    return CharacterDojang.model_validate(response)



from pydantic import BaseModel


class GuildRankingInfo(BaseModel):
    """유니온 랭킹 상세 정보

    Attributes:
        date: 랭킹 업데이트 일자 (KST, 일 단위 데이터로 시, 분은 일괄 0으로 표기)
        ranking: 길드 랭킹 순위
        guild_name: 길드 명
        world_name: 월드 명
        guild_level: 길드 레벨
        guild_master_name: 길드 마스터 캐릭터 명
        guild_mark: 길드 마크
        guild_point: 길드 포인트
    """

    date: str
    ranking: int
    guild_name: str
    world_name: str
    guild_level: int
    guild_master_name: str
    guild_mark: str
    guild_point: int


class GuildRanking(BaseModel):
    """길드 랭킹 정보

    Attributes:
        ranking: 길드 랭킹 정보
    """

    ranking: list[GuildRankingInfo]
